/* Generator is (c) James Ponder, 1997-2001 http://www.squish.net/generator/ */

/* user interface for svgalib (15/16 bit) */

/* this part of generator is 'slaved' from the ui-console.c file - only
   platform specific code is contained here */

#include <stdio.h>
#include <unistd.h>

#include "generator.h"

#include "uip.h"
#include "ui.h"
#include "ui-console.h"
#include "vdp.h"
#include "cpu68k.h"
#include "mem68k.h"

#define BANKSIZE 640*480*2
#define DISABLE_FLIPPING 1
#define ONLY_ONE_VIDEO_BANK 1

/*** forward reference declarations ***/

/*** static variables ***/

static uint8 uip_vga = 0;       /* flag for whether in VGA mode */
static uint8 uip_key[256];      /* keyboard state */
static uint8 uip_displaybanknum = 0;    /* view this one, write to other one */
static t_uipinfo *uip_uipinfo = NULL;   /* uipinfo */
static int uip_forceredshift = -1;      /* if set, forces red shift pos */
static int uip_forcegreenshift = -1;    /* if set, forces green shift pos */
static int uip_forceblueshift = -1;     /* if set, forces blue shift pos */
static uint8 video1[BANKSIZE];
static uint8 video2[BANKSIZE];

typedef struct
{
  uint8 address;
  uint8 vmus[6];
  uint8 vmu_count;
} t_controller;

t_controller controllers[4];

/*** Code ***/

static void uip_keyboardhandler(int scancode, int press)
{
}

int uip_init(t_uipinfo * uipinfo)
{
  uip_uipinfo = uipinfo;
  uipinfo->blueshift = 0;
  uipinfo->greenshift = 5;
  uipinfo->redshift = 10;
  uipinfo->linewidth = 640*2;
  uipinfo->screenmem0 = (uint8 *)&video1;
  uipinfo->screenmem1 = (uint8 *)&video2;
  uipinfo->screenmem_w = NULL;

  uipinfo->screenmem_w = uipinfo->screenmem0;
  vid_clear(0, 0, 0);
  ui_setupscreen();
#ifndef ONLY_ONE_VIDEO_BANK
  uipinfo->screenmem_w = uipinfo->screenmem1;
  vid_clear(0, 0, 0);
  ui_setupscreen();
  uipinfo->screenmem_w = uipinfo->screenmem0;
#endif
  return 0;
}

int uip_initjoysticks(void)
{
  int idx;
  uint32 unit;
  int controller_count = 0;

  for(idx = 0; idx < 4; idx++)
  {
    controllers[idx].address = 0;
    for(unit = 0; unit < 6; unit++)
    {
      controllers[idx].vmus[unit] = 0;
    }
    controllers[idx].vmu_count = 0;

    for(unit = 0; unit < 6; unit++)
    {
      if(maple_device_func(idx, unit) & MAPLE_FUNC_CONTROLLER)
      {
        controllers[idx].address = maple_addr(idx, unit);
	controller_count++;
      }
      else if(maple_device_func(idx, unit) & MAPLE_FUNC_LCD)
      {
        controllers[idx].vmus[controllers[idx].vmu_count++] = maple_addr(idx, unit);
      }
    }
  }
  return controller_count;
}

int uip_vgamode(void)
{
  return 0;
}

int uip_setcolourbits(int red, int green, int blue)
{
  uip_forceredshift = red;
  uip_forcegreenshift = green;
  uip_forceblueshift = blue;

  printf("RGB FORCED VALUES: %u:%u:%u\n", red, green, blue);
  return 0;
}

void uip_displaybank(int bank)
{
#ifdef DISABLE_FLIPPING
  vid_flip(bank);
  uip_uipinfo->screenmem_w = (uint8 *)vram_s;
#else
#ifdef ONLY_ONE_VIDEO_BANK
  sq_cpy(vram_s, &video1, BANKSIZE); 
#else
  if(bank == -1)
  {
    uip_displaybanknum ^= 1;
  }
  if(uip_displaybanknum == 0)
  {
    uip_uipinfo->screenmem_w = &video1;
    //memcpy(vram_s, &video2, 640*480*2);
    sq_cpy(vram_s, &video2, BANKSIZE); 
  }
  else
  {
    uip_uipinfo->screenmem_w = &video2;
    sq_cpy(vram_s, &video1, BANKSIZE); 
    //memcpy(vram_s, &video1, 640*480*2);
  }
#endif
#endif
}

void uip_clearscreen(void)
{
  //memset(uip_uipinfo->screenmem_w, 0, BANKSIZE);
  //vid_clear(0, 0, 0);
  vid_empty();
}

void uip_clearmiddle(void)
{
  int i;

  for (i = 0; i < 240; i++) {
    memset(uip_uipinfo->screenmem_w+
           (uip_uipinfo->linewidth) * (120 + i) + 160 * 2, 0, 2 * 320);
  }
}

/* uip_singlebank sets write address to the current display screen */

void uip_singlebank(void)
{
#ifdef DISABLE_FLIPPING
  vid_flip(-1);
  uip_uipinfo->screenmem_w = (uint8 *)vram_s;
#endif
}

/* uip_doublebank sets write address to hidden display screen */

void uip_doublebank(void)
{
#ifdef DISABLE_FLIPPING
  vid_flip(-1);
  uip_uipinfo->screenmem_w = (uint8 *)vram_s;
#endif
}

void uip_textmode(void)
{
}

int uip_checkkeyboard(void)
{
  int i;
  int pad = 0;
  cont_cond_t cond;
  int quit = 1;

  for(i = 0; i < 4; i++)
  {
    if(controllers[i].address == 0 || pad > 1)
    {
      continue;
    }
    cont_get_cond(controllers[i].address, &cond);
    mem68k_cont[pad].up = !(cond.buttons & CONT_DPAD_UP);
    mem68k_cont[pad].right = !(cond.buttons & CONT_DPAD_RIGHT);
    mem68k_cont[pad].down = !(cond.buttons & CONT_DPAD_DOWN);
    mem68k_cont[pad].left = !(cond.buttons & CONT_DPAD_LEFT);
    mem68k_cont[pad].start = !(cond.buttons & CONT_START);
    mem68k_cont[pad].a = !(cond.buttons & CONT_X);
    mem68k_cont[pad].b = !(cond.buttons & CONT_A);
    mem68k_cont[pad].c = !(cond.buttons & CONT_B);
    pad++;
    if(quit)
    {
      quit = !(cond.buttons & CONT_Y);
    }

    if(cond.ltrig && !cond.rtrig)
    {
      ui_fullscreen ^= 1; 
      ui_clearnext = 2;
    }
    else if(!cond.ltrig && cond.rtrig)
    {
      sound_stop();
      gen_reset();
      ui_clearnext = 2;
      sound_start();
    }
    else if(cond.rtrig && cond.ltrig && !quit)
    {
      quit = 1;
    }
  }

  return quit;
}

void uip_vsync(void)
{
#ifdef DISABLE_FLIPPING
  vid_waitvbl();
#endif
}

unsigned int uip_whichbank(void)
{
  /* returns 0 or 1 - the bank being VIEWED */
  return uip_displaybanknum;
}

uint8 uip_getchar(void)
{
  return 0;
}

int ui_check_cond(uint32 flags)
{
	int i;
	cont_cond_t cond;
	for(i = 0; i < 4; i++)
	{
		if(controllers[i].address == 0)
		{
			continue;
		}
		cont_get_cond(controllers[i].address, &cond);
		if(!(cond.buttons & flags))
		{
			return 1;
		}
	}
	return 0;
}
